@@ -1,6 +1,6 @@
 /*
     orzwinmenu
-    copyright (c) 1998-2008 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2009 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -171,7 +171,7 @@ orz_winmenu_change (OrzMdi     *mdi,
   if (winmenu && winmenu->mdi == mdi && winmenu->menu_shell && winmenu->glist)
     {
       gchar *label;
-      gint i;
+      GList *glist;
       GtkWidget *child;
 
       child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (mdi), page_num);
@@ -181,15 +181,11 @@ orz_winmenu_change (OrzMdi     *mdi,
       label = g_filename_to_utf8 (orz_mdi_get_file (mdi, page_num), -1,
                                                             NULL, NULL, NULL);
 #endif /* not GLIB_CHECK_VERSION(2,6,0) */
-      for (i = g_list_length (winmenu->glist) - 1; i >= 0; i--)
-        {
-          GtkWidget *menu_item;
-
-          menu_item = g_list_nth_data (winmenu->glist, i);
-          if (g_object_get_data (G_OBJECT (menu_item), "user_data") == child)
-            gtk_label_set_text (GTK_LABEL (gtk_bin_get_child
-                                                (GTK_BIN (menu_item))), label);
-        }
+      for (glist = g_list_first (winmenu->glist);
+                                            glist; glist = g_list_next (glist))
+        if (g_object_get_data (G_OBJECT (glist->data), "user_data") == child)
+          gtk_label_set_text (GTK_LABEL (gtk_bin_get_child
+                                            (GTK_BIN (glist->data))), label);
       g_free (label);
     }
 }
@@ -203,13 +199,14 @@ orz_winmenu_reorder (OrzMdi     *mdi,
 {
   if (winmenu && winmenu->mdi == mdi && winmenu->menu_shell && winmenu->glist)
     {
-      gint i;
+      GList *glist;
       GtkWidget *child, *menu_item;
 
       child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (mdi), page_num);
-      for (i = g_list_length (winmenu->glist) - 1; i >= 0; i--)
+      for (glist = g_list_first (winmenu->glist);
+                                            glist; glist = g_list_next (glist))
         {
-          menu_item = g_list_nth_data (winmenu->glist, i);
+          menu_item = glist->data;
           if (g_object_get_data (G_OBJECT (menu_item), "user_data") == child)
             {
               g_object_ref (G_OBJECT (menu_item));
@@ -219,10 +216,9 @@ orz_winmenu_reorder (OrzMdi     *mdi,
               break;
             }
         }
-      if (i >= 0)
+      if (glist)
         {
-          gint length, position;
-          GList *glist;
+          gint i, length, position;
 
           glist = gtk_container_get_children
                                         (GTK_CONTAINER (winmenu->menu_shell));
@@ -263,13 +259,16 @@ orz_winmenu_remove (OrzMdi     *mdi,
 {
   if (winmenu && winmenu->mdi == mdi && winmenu->menu_shell && winmenu->glist)
     {
-      gint i;
-      GtkWidget *child, *menu_item;
+      GList *glist;
+      GtkWidget *child;
 
       child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (mdi), page_num);
-      for (i = g_list_length (winmenu->glist) - 1; i >= 0; i--)
+      for (glist = g_list_first (winmenu->glist);
+                                            glist; glist = g_list_next (glist))
         {
-          menu_item = g_list_nth_data (winmenu->glist, i);
+          GtkWidget *menu_item;
+
+          menu_item = glist->data;
           if (g_object_get_data (G_OBJECT (menu_item), "user_data") == child)
             {
               gtk_container_remove (GTK_CONTAINER (winmenu->menu_shell),
@@ -294,12 +293,12 @@ orz_winmenu_remove (OrzMdi     *mdi,
             }
           if (GTK_IS_MENU (winmenu->menu_shell))
             {
-              GList *glist;
-
               glist = gtk_container_get_children
                                         (GTK_CONTAINER (winmenu->menu_shell));
               if (g_list_length (glist) <= 0)
                 {
+                  GtkWidget *menu_item;
+
                   menu_item = gtk_menu_get_attach_widget
                                             (GTK_MENU (winmenu->menu_shell));
                   if (menu_item)
@@ -320,22 +319,20 @@ orz_winmenu_switch (OrzMdi     *mdi,
 {
   if (winmenu && winmenu->mdi == mdi && winmenu->menu_shell && winmenu->glist)
     {
-      gint i;
-      GtkWidget *child, *menu_item;
+      GList *glist;
+      GtkWidget *child;
 
       child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (mdi), page_num);
-      for (i = g_list_length (winmenu->glist) - 1; i >= 0; i--)
-        {
-          menu_item = g_list_nth_data (winmenu->glist, i);
-          if (g_object_get_data (G_OBJECT (menu_item), "user_data") == child)
-            {
-              g_object_set_data (G_OBJECT (menu_item), "user_data", NULL);
-              gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item),
-                                                                        TRUE);
-              g_object_set_data (G_OBJECT (menu_item), "user_data", child);
-              break;
-            }
-        }
+      for (glist = g_list_first (winmenu->glist);
+                                            glist; glist = g_list_next (glist))
+        if (g_object_get_data (G_OBJECT (glist->data), "user_data") == child)
+          {
+            g_object_set_data (G_OBJECT (glist->data), "user_data", NULL);
+            gtk_check_menu_item_set_active
+                                    (GTK_CHECK_MENU_ITEM (glist->data), TRUE);
+            g_object_set_data (G_OBJECT (glist->data), "user_data", child);
+            break;
+          }
     }
 }
 
