@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2008 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2009 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -217,21 +217,41 @@ add_dialog (MmaidWindow *mmaid)
   combo_album = gtk_combo_box_entry_new_text ();
   combo_comment = gtk_combo_box_entry_new_text ();
   combo_genre = gtk_combo_box_entry_new_text ();
-  for (i = g_list_length (glist_title) - 1; i >= 0; i--)
-    gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo_title),
-                                        g_list_nth_data (glist_title, i));
-  for (i = g_list_length (glist_artist) - 1; i >= 0; i--)
-    gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo_artist),
-                                        g_list_nth_data (glist_artist, i));
-  for (i = g_list_length (glist_album) - 1; i >= 0; i--)
-    gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo_album),
-                                        g_list_nth_data (glist_album, i));
-  for (i = g_list_length (glist_comment) - 1; i >= 0; i--)
-    gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo_comment),
-                                        g_list_nth_data (glist_comment, i));
-  for (i = g_list_length (glist_genre) - 1; i >= 0; i--)
-    gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo_genre),
-                                        g_list_nth_data (glist_genre, i));
+  for (glist_title = g_list_first (glist_title); glist_title;
+            glist_title = g_list_delete_link (glist_title, glist_title))
+    {
+      gtk_combo_box_append_text (GTK_COMBO_BOX (combo_title),
+                                                        glist_title->data);
+      g_free (glist_title->data);
+    }
+  for (glist_artist = g_list_first (glist_artist); glist_artist;
+            glist_artist = g_list_delete_link (glist_artist, glist_artist))
+    {
+      gtk_combo_box_append_text (GTK_COMBO_BOX (combo_artist),
+                                                        glist_artist->data);
+      g_free (glist_artist->data);
+    }
+  for (glist_album = g_list_first (glist_album); glist_album;
+            glist_album = g_list_delete_link (glist_album, glist_album))
+    {
+      gtk_combo_box_append_text (GTK_COMBO_BOX (combo_album),
+                                                        glist_album->data);
+      g_free (glist_album->data);
+    }
+  for (glist_comment = g_list_first (glist_comment); glist_comment;
+            glist_comment = g_list_delete_link (glist_comment, glist_comment))
+    {
+      gtk_combo_box_append_text (GTK_COMBO_BOX (combo_comment),
+                                                        glist_comment->data);
+      g_free (glist_comment->data);
+    }
+  for (glist_genre = g_list_first (glist_genre); glist_genre;
+            glist_genre = g_list_delete_link (glist_genre, glist_genre))
+    {
+      gtk_combo_box_append_text (GTK_COMBO_BOX (combo_genre),
+                                                        glist_genre->data);
+      g_free (glist_genre->data);
+    }
 #else /* not GTK_CHECK_VERSION(2,4,0) */
   combo_title = gtk_combo_new ();
   combo_artist = gtk_combo_new ();
@@ -239,15 +259,35 @@ add_dialog (MmaidWindow *mmaid)
   combo_comment = gtk_combo_new ();
   combo_genre = gtk_combo_new ();
   if (glist_title)
-    gtk_combo_set_popdown_strings (GTK_COMBO (combo_title), glist_title);
+    {
+      gtk_combo_set_popdown_strings (GTK_COMBO (combo_title), glist_title);
+      g_list_find_custom (glist_title, (GFunc)g_free, NULL);
+      g_list_free (glist_title);
+    }
   if (glist_artist)
-    gtk_combo_set_popdown_strings (GTK_COMBO (combo_artist), glist_artist);
+    {
+      gtk_combo_set_popdown_strings (GTK_COMBO (combo_artist), glist_artist);
+      g_list_find_custom (glist_artist, (GFunc)g_free, NULL);
+      g_list_free (glist_artist);
+    }
   if (glist_album)
-    gtk_combo_set_popdown_strings (GTK_COMBO (combo_album), glist_album);
+    {
+      gtk_combo_set_popdown_strings (GTK_COMBO (combo_album), glist_album);
+      g_list_find_custom (glist_album, (GFunc)g_free, NULL);
+      g_list_free (glist_album);
+    }
   if (glist_comment)
-    gtk_combo_set_popdown_strings (GTK_COMBO (combo_comment), glist_comment);
+    {
+      gtk_combo_set_popdown_strings (GTK_COMBO (combo_comment), glist_comment);
+      g_list_find_custom (glist_comment, (GFunc)g_free, NULL);
+      g_list_free (glist_comment);
+    }
   if (glist_genre)
-    gtk_combo_set_popdown_strings (GTK_COMBO (combo_genre), glist_genre);
+    {
+      gtk_combo_set_popdown_strings (GTK_COMBO (combo_genre), glist_genre);
+      g_list_find_custom (glist_genre, (GFunc)g_free, NULL);
+      g_list_free (glist_genre);
+    }
   gtk_combo_set_value_in_list (GTK_COMBO (combo_title), FALSE, FALSE);
   gtk_combo_set_value_in_list (GTK_COMBO (combo_artist), FALSE, FALSE);
   gtk_combo_set_value_in_list (GTK_COMBO (combo_album), FALSE, FALSE);
@@ -324,21 +364,6 @@ add_dialog (MmaidWindow *mmaid)
   g_signal_connect (G_OBJECT (GTK_COMBO (combo_genre)->entry), "changed",
                                     G_CALLBACK (add_dialog_changed), &adddlg);
 #endif /* not GTK_CHECK_VERSION(2,4,0) */
-  for (i = g_list_length (glist_title) - 1; i >= 0; i--)
-    g_free (g_list_nth_data (glist_title, i));
-  for (i = g_list_length (glist_artist) - 1; i >= 0; i--)
-    g_free (g_list_nth_data (glist_artist, i));
-  for (i = g_list_length (glist_album) - 1; i >= 0; i--)
-    g_free (g_list_nth_data (glist_album, i));
-  for (i = g_list_length (glist_comment) - 1; i >= 0; i--)
-    g_free (g_list_nth_data (glist_comment, i));
-  for (i = g_list_length (glist_genre) - 1; i >= 0; i--)
-    g_free (g_list_nth_data (glist_genre, i));
-  g_list_free (glist_title);
-  g_list_free (glist_artist);
-  g_list_free (glist_album);
-  g_list_free (glist_comment);
-  g_list_free (glist_genre);
   /* ja:文字符号化方式 */
   adddlg.charset = orz_charset_new ();
   orz_charset_add_candidate (ORZ_CHARSET (adddlg.charset), charset_list);
