@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2008 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2009 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -268,9 +268,9 @@ command_saveas (GtkWidget *widget,
                 gpointer   user_data)
 {
   gchar *file, *root = NULL, *charset, *name, *path, *tmp, **ary, *title;
-  gint playlist, i, page_num;
+  gint i, playlist, page_num;
   guint filekind;
-  GList *glist = NULL;
+  GList *gl, *glist = NULL;
   GtkStockItem stock_item;
   GtkWidget *dialog, *hbox0, *hbox1, *vbox;
   GtkWidget *combo0, *combo1, *label0, *label1;
@@ -400,11 +400,10 @@ command_saveas (GtkWidget *widget,
   /* ja:コンボボックス */
 #if GTK_CHECK_VERSION(2,4,0)
   combo0 = gtk_combo_box_new_text ();
-  for (i = g_list_length (glist) - 1; i >= 0; i--)
+  for (gl = g_list_first (glist); gl; gl = g_list_next (gl))
     {
-      gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo0),
-                                                g_list_nth_data (glist, i));
-      if (g_strfilecmp (g_list_nth_data (glist, i), root) == 0)
+      gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo0), gl->data);
+      if (g_strfilecmp (gl->data, root) == 0)
         gtk_combo_box_set_active (GTK_COMBO_BOX (combo0), 0);
     }
   g_signal_connect (G_OBJECT (combo0), "changed",
@@ -419,8 +418,7 @@ command_saveas (GtkWidget *widget,
   g_signal_connect (G_OBJECT (GTK_COMBO (combo0)->entry), "changed",
                                     G_CALLBACK (command_save_changed), dialog);
 #endif /* not GTK_CHECK_VERSION(2,4,0) */
-  for (i = g_list_length (glist) - 1; i >= 0; i--)
-    g_free (g_list_nth_data (glist, i));
+  g_list_find_custom (glist, (GFunc)g_free, NULL);
   g_list_free (glist);
   /* ja:文字符号化方式 */
   combo1 = orz_charset_new ();
