@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2008 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2009 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -1021,18 +1021,15 @@ mm_list_set_cursor (MmList     *list,
 
   if (c != MM_LIST_INVALID)
     {
-      gint i;
       GList *glist;
 
       glist = gtk_tree_view_get_columns (GTK_TREE_VIEW (list));
-      for (i = g_list_length (glist) - 1; i >= 0; i--)
-        {
-
-          column = g_list_nth_data (glist, i);
-          if (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (column),
+      for (glist = g_list_first (glist);
+                            glist; glist = g_list_delete_link (glist, glist))
+        if (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (glist->data),
                                                             "user_data")) == c)
-            break;
-        }
+          break;
+      g_list_free (glist);
     }
   if (n >= 0)
     {
