@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2008 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2009 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -263,9 +263,9 @@ root_dialog (MmaidWindow *mmaid)
       }
 #if GTK_CHECK_VERSION(2,4,0)
   rtdlg.combo = gtk_combo_box_new_text ();
-  for (i = g_list_length (glist) - 1; i >= 0; i--)
-    gtk_combo_box_prepend_text (GTK_COMBO_BOX (rtdlg.combo),
-                                                g_list_nth_data (glist, i));
+  for (glist = g_list_first (glist);
+                            glist; glist = g_list_delete_link (glist, glist))
+    gtk_combo_box_append_text (GTK_COMBO_BOX (rtdlg.combo), glist->data);
   gtk_combo_box_set_active (GTK_COMBO_BOX (rtdlg.combo), 0);
   g_signal_connect (G_OBJECT (rtdlg.combo), "changed",
                                     G_CALLBACK (root_dialog_changed), &rtdlg);
@@ -286,10 +286,9 @@ root_dialog (MmaidWindow *mmaid)
                                     G_CALLBACK (root_dialog_activate), &rtdlg);
   g_signal_connect (G_OBJECT (GTK_COMBO (rtdlg.combo)->entry), "changed",
                                     G_CALLBACK (root_dialog_changed), &rtdlg);
-#endif /* not GTK_CHECK_VERSION(2,4,0) */
-  for (i = g_list_length (glist) - 1; i >= 0; i--)
-    g_free (g_list_nth_data (glist, i));
+  g_list_find_custom (glist, (GFunc)g_free, NULL);
   g_list_free (glist);
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
   /* ja:エントリー */
   rtdlg.entry = gtk_entry_new ();
   g_signal_connect (G_OBJECT (rtdlg.entry), "changed",
